LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use IEEE.std_logic_textio.all;
use STD.textio.all;
use IEEE.numeric_std.all;
use ieee.math_real.all;

entity FT2232H_minimal_bench is
end FT2232H_minimal_bench;

architecture Behavioral of FT2232H_minimal_bench is

component FT2232H_minimal_D is Port(
	CLK100: in std_logic;
	-- FT2232H
	D: out std_logic_vector(7 downto 0);
	RXF: in std_logic;
	TXE: in std_logic;
	RD: out std_logic;
	WR: out std_logic;
	SIWU: out std_logic;
	CLKOUT: in std_logic; --60MHz FTDI
	OE: out std_logic;
	PWRSAV: out std_logic);
end component;

signal clk100: std_logic:='0';
signal CLKOUT: std_logic:='0';
signal TXE: std_logic:='0';
type Rom_SR is array (39 downto 0) of std_logic_vector(7 downto 0);
signal bench_SR, bench_Packet_RCV, Refpacket: Rom_SR;
signal D: std_logic_vector(7 downto 0):=(others => '0');
signal WR : std_logic:='0';
signal bench_D_buff: std_logic_vector(7 downto 0):=(others => '0');
signal bench_sync: std_logic:='0';
signal bench_sync_counter: unsigned(7 downto 0):=(others => '0');
signal bench_new_packet: std_logic:='0';
signal bench_fail: std_logic:='0';
signal bench_RCV: std_logic:='0';
signal bench_FT2232H_not_full: std_logic:='0';
signal rand_num_1k : integer := 500;


begin
	
uut: FT2232H_minimal_D port map(
	CLK100 => clk100,
	-- FT2232H
	D => D,
	RXF => '1',
	TXE => TXE,
	RD => open,
	WR => WR,
	SIWU => open,
	CLKOUT => CLKOUT,
	OE => open,
	PWRSAV => open);

clk100 <= not clk100 after 5ns;
CLKOUT <= not CLKOUT after 8.333ns;
bench_FT2232H_not_full <= not bench_FT2232H_not_full after rand_num_1k*1ns;

process
begin
	wait until rising_edge(CLKOUT);
	for I in 0 to 39 loop
		if (I = 0) then
			Refpacket(I) <= x"01";
		else
			Refpacket(I) <= std_logic_vector(to_unsigned((I*2),8));
		end if;
	end loop;
end process;

process
    variable seed1, seed2: positive; -- seed values for random generator
    variable rand: real; -- random real-number value in range 0 to 1.0  
	begin
    uniform(seed1, seed2, rand); -- generate random number
    rand_num_1k <= integer(rand*1000.0); -- rescale to 0..1000, convert integer part
    wait for 100 ns;
end process;

process
begin
	wait until rising_edge(CLKOUT);
	TXE <= '1';
	if bench_FT2232H_not_full = '1' then
		TXE <= '0';
	end if;
end process;

process begin	
	wait until rising_edge(CLKOUT);
	bench_fail <= '0';
	bench_RCV <= '0';

	if WR = '0' and TXE = '0' then
		bench_RCV <= '1';
		bench_SR <= D & bench_SR(39 downto 1);
		
	end if;
	bench_new_packet <= '0';
	if bench_RCV = '1' then
		bench_sync_counter <= bench_sync_counter +1;
		if bench_SR = Refpacket then
			bench_new_packet <= '1';
			bench_Packet_RCV <= bench_SR;
			bench_sync_counter <= x"00";
		end if;
	end if;
	if bench_sync_counter > 39 then
		bench_fail <= '1';
	end if;
end process;					   

end Behavioral;